<template>
  <div class="main">
    <div class="inner" ref="wraper">
      <a-spin class="" :spinning="spinning" tip="拼命加载中...">
        <a-card>
          <advance-table
            ref="table"
            class="doc-center-table"
            :data-source="dataSource"
            :columns="columns"
            :loading="loading"
            title=""
            :row-key="(record, index) => record.subCode + '_' + index"
            @search="onSearch"
            @refresh="onRefresh"
            @reset="onReset"
            :format-conditions="true"
            :scroll="{ x: 1360, y }"
            :pagination="{
              current: pageCurr,
              pageSize: pageSize,
              total: total,
              showSizeChanger: true,
              showLessItems: true,
              showQuickJumper: true,
              pageSizeOptions: ['10', '20', '50', '100'],
              showTotal: (total, range) =>
                `第 ${range[0]}-${range[1]} 条，总计 ${total} 条`,
              onChange: onPageChange,
              onShowSizeChange: onSizeChange,
            }"
          >
            <template slot="dataIndex" slot-scope="{ index }">
              {{ index + 1 }}
            </template>
            <template slot="title">
              <span class="title">物料中心</span>
              <a-space class="operator" style="margin-left: 1em">
                <a-upload
                  v-if="canUpload"
                  :before-upload="beforeUpload"
                  :showUploadList="false"
                  @change="uploadChange"
                  accept=".zip"
                >
                  <a-button type="primary">新增</a-button>
                </a-upload>
              </a-space>
            </template>
            <template slot="pictureUrl" slot-scope="{ record }">
              <div class="img-wraper">
                <image-view
                  v-if="record.pictureUrl"
                  :url="webUrl + record.pictureUrl"
                ></image-view>
              </div>
            </template>
            <template slot="action" slot-scope="{ record }">
              <a @click="downloadLogs(record)">下载日志</a>
              <a-divider type="vertical"></a-divider>
              <a @click="addLink(record)">增加关联</a>
              <template v-if="record.mproductHistorys.length">
                <a-divider type="vertical"></a-divider>
                <a @click="removeLink(record)">解除关联</a>
              </template>
              <a-divider type="vertical"></a-divider>
              <a @click="panelShow(record)">附件</a>
              <template v-if="record.rootModel">
                <a-divider type="vertical"></a-divider>
                <a @click="goHistory(record)">历史</a>
              </template>
              <!-- <template v-if="record.dwgUrl">
                <a-divider type="vertical"></a-divider>
                <a @click="dwgReview(record.dwgUrl)">预览</a>
                <template>
                  <a-divider type="vertical"></a-divider>
                  <a @click="downloadLog(record)">下载</a>
                </template>
              </template> -->
            </template>
          </advance-table>
        </a-card>
      </a-spin>
    </div>
    <!-- 关联 -->
    <a-modal
      :visible="editShow"
      :footer="null"
      :title="editTitle"
      :destroyOnClose="true"
      :width="1200"
      :maskClosable="false"
      @cancel="cancel"
    >
      <edit-link
        :com-data="component"
        :prod-list="prodList"
        :is-add="isAdd"
        @close="cancel"
      ></edit-link>
    </a-modal>
    <!-- 弹窗 解析结果 -->
    <a-modal
      :visible="resShow"
      title="解析结果"
      :destroyOnClose="true"
      :maskClosable="false"
      :width="800"
      :footer="null"
      @cancel="resCancel"
    >
      <parts-upload :list="resList" @success="resOk"></parts-upload>
    </a-modal>
    <!-- 提交流程 -->
    <a-modal
      :visible="submitShow"
      title="提交审核"
      :destroyOnClose="true"
      :maskClosable="false"
      :width="500"
      :footer="null"
      @cancel="cancelSubmit"
    >
      <submit-form
        :title="title"
        :list="resList"
        @success="successSubmit"
        @cancel="cancelSubmit"
      ></submit-form>
    </a-modal>
    <!-- 附件 -->
    <a-modal
      :visible="fileListShow"
      title="附件"
      :destroyOnClose="true"
      :maskClosable="false"
      :width="850"
      :footer="null"
      @cancel="fileListCancel"
    >
      <files-table
        :list="fileList"
        :info="info"
        :sub-model="currentObj.subModel"
        :type="13"
      ></files-table>
      <div class="footer">
        <a-space>
          <a-button @click="fileListCancel">关闭</a-button>
          <a-button type="primary" v-if="canUpload" @click="fileUploadShow"
            >上传附件</a-button
          >
        </a-space>
      </div>
    </a-modal>
    <!-- 上传附件 -->
    <a-modal
      :visible="fileUploadVisible"
      title="附件上传"
      :destroyOnClose="true"
      :maskClosable="false"
      :width="600"
      @cancel="fileUploadCancel"
      @ok="fileUploadChange"
    >
      <div class="type-grp">
        <a-radio-group v-model="uploadType" button-style="solid">
          <a-radio-button value="normal"> 普通附件 </a-radio-button>
          <a-radio-button value="pic"> 图片 </a-radio-button>
          <a-radio-button value="dwg"> 图纸 </a-radio-button>
        </a-radio-group>
      </div>
      <a-upload
        :before-upload="beforeUpload"
        @change="uploadListChange"
        multiple
      >
        <!-- :showUploadList="false" -->
        <a-button type="primary">选择附件</a-button>
      </a-upload>
    </a-modal>
    <a-modal
      :visible="errorVisible"
      :footer="null"
      :width="1200"
      title="名称或型号命名不规范"
      :destroyOnClose="true"
      :maskClosable="false"
      @cancel="errorVisible = false"
    >
      <a-table
        ref="aTable"
        size="small"
        :scroll="{ y: 500 }"
        bordered
        :columns="errorTbl2.columns"
        :data-source="errorTbl2.dataSource"
        :pagination="false"
        rowKey="num"
      ></a-table>
    </a-modal>
    <download-logs
      :visible.sync="downloadlogVisible"
      v-if="downloadlogVisible"
      :type="13"
      :oprate-info="oprateInfo"
    ></download-logs>
  </div>
</template>

<script>
import PartsUpload from "@/pages/components/PartsUpload";
import AdvanceTable from "@/components/table/advance/AdvanceTable";
import ImageView from "@/pages/components/ImageView";
import EditLink from "./editLink";
import SubmitForm from "./SubmitForm";
import FilesTable from "@/pages/components/filesTable";
import DownloadLogs from "@/pages/components/downloadLogs";

import getWebUrl from "@/assets/js/tools/getWebUrl";
import {
  uploadOthers,
  picOrDwg,
  zipParse,
  getFileList,
  addMaterial,
} from "./apis";
import { dwgReview } from "@/pages/workplace/apis";
// import { downloadLog } from "@/pages/system/logs/apis";
import { mapGetters } from "vuex";
import checkPermit from "@/assets/js/tools/checkPermit";
import PERMITS from "@/assets/js/const/const_permits";

import createWs from "@/assets/js/websocket";
const WSMixin = createWs("material");
export default {
  name: "",
  mixins: [WSMixin],
  data() {
    return {
      oprateInfo: "",
      downloadlogVisible: false,
      errorVisible: false,
      errorTbl2: {
        columns: [
          {
            title: "子件编码",
            dataIndex: "subCode",
            align: "center",
          },
          {
            title: "子件型号",
            dataIndex: "subModel",
            align: "center",
          },
          {
            title: "子件名称",
            dataIndex: "subName",
            align: "center",
          },
          {
            title: "问题描述",
            dataIndex: "irregularDesc",
            align: "center",
          },
        ],
        dataSource: [],
      },
      uploadType: "normal",
      fileUploadVisible: false,
      uploadFileList: [],
      info: {},
      fileListShow: false,
      fileList: [],
      resShow: false,
      submitShow: false,
      currentObj: {},
      title: "",
      resList: [],
      // 是否为新增关联 false表示为删除关联
      isAdd: true,
      component: null,
      prodList: [],
      editShow: false,
      selectedRowKeys: [],
      selectedRows: [],
      spinning: false,
      loading: false,
      pageCurr: 1,
      pageSize: 10,
      total: 0,
      y: 400,
      update: -1,
      webUrl: getWebUrl(),
      conditions: {},
      columns: [
        {
          fixed: "left",
          title: "序号",
          dataIndex: "dataIndex",
          key: "dataIndex",
          align: "center",
          width: 60,
          noSearch: true,
          scopedSlots: { customRender: "dataIndex" },
        },
        // {
        //   title: "类别",
        //   dataIndex: "category",
        //   key: "category",
        //   align: "center",
        //   width: 80,
        //   searchAble: true,
        // },
        {
          title: "编码",
          dataIndex: "subCode",
          key: "subCode",
          align: "center",
          width: 130,
          searchAble: true,
        },
        {
          title: "型号",
          dataIndex: "subModel",
          key: "subModel",
          searchAble: true,
          align: "center",
        },
        {
          title: "名称",
          dataIndex: "subName",
          key: "subName",
          align: "center",
          searchAble: true,
        },
        {
          title: "单位",
          dataIndex: "unit",
          key: "unit",
          align: "center",
          width: 80,
          noSearch: true,
        },
        // {
        //   title: "生产商",
        //   dataIndex: "producer",
        //   key: "producer",
        //   align: "center",
        //   width: 80,
        //   noSearch: true,
        // },
        // {
        //   title: "元件编号/料厚",
        //   dataIndex: "thickness",
        //   key: "thickness",
        //   align: "center",
        //   noSearch: true,
        //   width: 180,
        // },
        // {
        //   title: "表面处理/物料详情",
        //   dataIndex: "surfaceDetail",
        //   key: "surfaceDetail",
        //   align: "center",
        //   noSearch: true,
        //   width: 180,
        // },
        // {
        //   title: "备注",
        //   dataIndex: "notes",
        //   key: "notes",
        //   align: "center",
        //   noSearch: true,
        // },
        {
          title: "图片",
          dataIndex: "pictureUrl",
          key: "pictureUrl",
          align: "center",
          noSearch: true,
          width: 200,
          scopedSlots: { customRender: "pictureUrl" },
        },
        // {
        //   title: "上传人",
        //   dataIndex: "upUser",
        //   key: "upUser",
        //   align: "center",
        //   noSearch: true,
        // },
        // {
        //   title: "创建时间",
        //   dataIndex: "createDate",
        //   key: "createDate",
        //   align: "center",
        //   width: 160,
        //   noSearch: true,
        // },
        {
          title: "操作",
          dataIndex: "operation",
          key: "operation",
          align: "center",
          fixed: "right",
          width: 270,
          noSearch: true,
          scopedSlots: { customRender: "action" },
        },
      ],
      dataSource: [],
      linkColumns: [
        {
          title: "母料型号",
          dataIndex: "parentModel",
          // width: 200,
          align: "center",
          ellipsis: true,
        },
        {
          title: "子件名称",
          dataIndex: "subName",
          width: 200,
          align: "center",
          ellipsis: true,
        },
        {
          title: "关联/替换",
          dataIndex: "linkType",
          width: 100,
          align: "center",
          scopedSlots: { customRender: "linkType" },
        },
      ],
    };
  },
  computed: {
    editTitle() {
      return this.isAdd ? "批量增加关联关系" : "批量解除关联关系";
    },
    ...mapGetters("setting", ["affixed"]),
    ...mapGetters("account", ["permits"]),
    canUpload() {
      return checkPermit(PERMITS.uploadBom, this.permits);
    },
  },
  components: {
    AdvanceTable,
    ImageView,
    EditLink,
    PartsUpload,
    SubmitForm,
    FilesTable,
    DownloadLogs,
  },
  methods: {
    downloadLogs(record) {
      // console.log(record);
      let { id } = record;
      this.oprateInfo = id;
      this.downloadlogVisible = true;
    },
    onSearch(conditions, searchOptions) {
      // console.log(conditions);
      // console.log(searchOptions);
      this.pageCurr = 1;
      this.conditions = conditions;
      this.sendMessage();
    },
    onPageChange(page, pageSize) {
      this.pageCurr = page;
      this.pageSize = pageSize;
      this.sendMessage();
    },
    onSizeChange(current, size) {
      this.pageCurr = 1;
      this.pageSize = size;
      this.sendMessage();
    },
    onRefresh(conditions) {
      this.conditions = conditions;
      this.sendMessage();
    },
    onReset(conditions) {
      this.conditions = conditions;
      this.sendMessage();
    },
    onWSOpen() {
      this.$nextTick(() => {
        this.sendMessage();
      });
    },
    sendMessage() {
      if (!this.isWSOpen) {
        return false;
      }
      const { pageCurr, pageSize, conditions } = this;
      let params = {};
      Object.keys(conditions).forEach((v) => {
        params[v] = conditions[v];
      });
      let data = {
        pageSize,
        pageCurr,
        ...params,
      };
      // console.log("=====9=", data, JSON.stringify(data));
      this.SOCKET.send(JSON.stringify(data));
    },
    onWSMessage(res) {
      res = JSON.parse(res.data);
      // // console.log(res, "=====111data");
      this.searchData(res);
    },
    searchData(res) {
      if (res) {
        // res = res.data;
        // console.log(res);
        let data = [];
        let total = 0;
        if (res.code && res.data) {
          data = res.data2.list.map((v) => {
            const model = v.subModel;
            const reg = /(.*)(-|_)A/;
            const rootModel = reg.test(model) ? model.match(reg)[1] : "";
            return {
              ...v,
              rootModel,
            };
          });
          total = res.data2.total;
        }
        this.dataSource = data;
        this.total = total;
        if (-1 == this.update) {
          this.update = Math.random();
        }
      }
    },
    // 添加关联
    addLink(obj) {
      // console.log("addlink", obj);
      const { id, products } = obj;
      if (!id) {
        return false;
      }
      this.isAdd = true;
      this.component = obj;
      this.prodList = products;
      this.editShow = true;
    },
    removeLink(obj) {
      this.isAdd = false;
      this.component = obj;
      this.prodList = obj.products;
      this.editShow = true;
    },
    // 删除散装件
    remove(obj) {
      // console.log("remove", obj);
      const { id, subName } = obj;
      this.resList = [{ componentId: id }];
      this.title = `删除散装件 ${subName}`;
      this.submitShow = true;
    },
    cancel() {
      this.editShow = false;
    },
    resize() {
      setTimeout(() => {
        this.update = Math.random();
      }, 200);
    },
    beforeUpload() {
      return false;
    },
    uploadChange(data) {
      let loading = this.$layer.loading();
      const formData = new FormData();
      formData.append("file", data.file);
      zipParse(formData)
        .then((res) => {
          this.$layer.close(loading);
          let rs = res.data;
          if (rs.code == 1 && rs.data) {
            this.resList = rs.data2;
            this.title = rs.data3;
            rs.data2[0].fileUrl = rs.msg;
            this.resShow = true;
            this.$message.success("解析成功");
          } else {
            if (rs.msg == "名称或型号命名不规范") {
              this.errorTbl2.dataSource = rs.data2;
              this.errorVisible = true;
            } else {
              this.$message.error(rs.msg);
            }
          }
        })
        .catch((error) => {
          this.$layer.close(loading);
          console.log(error);
        });
    },
    fileUploadShow() {
      this.uploadFileList = [];
      this.fileUploadVisible = true;
    },
    fileUploadCancel() {
      this.fileUploadVisible = false;
    },
    uploadListChange(data) {
      this.uploadFileList = data.fileList;
    },
    fileUploadChange(data) {
      if (!this.uploadFileList.length) {
        this.$message.error("请选择要上传的附件");
        return false;
      }
      let loading = this.$layer.loading();
      const formData = new FormData();
      const { id, subModel, subCode } = this.currentObj;
      const obj = {
        id,
        subCode,
        subModel,
      };
      let dwgFileName = subModel.toLowerCase() + ".dwg";
      formData.append("materialStr", JSON.stringify(obj));
      this.uploadFileList.forEach((v, i) => {
        formData.append("file", v.originFileObj);
      });
      if (
        this.uploadType == "normal" &&
        this.uploadFileList.some((v) => v.name.toLowerCase() == dwgFileName)
      ) {
        this.$message.error("所选附件类型不匹配");
        this.$layer.close(loading);
        return false;
      }
      if (
        this.uploadType == "dwg" &&
        this.uploadFileList.some((v) => v.name.toLowerCase() != dwgFileName)
      ) {
        this.$message.error("所选附件类型不匹配");
        this.$layer.close(loading);
        return false;
      }
      let upload = this.uploadType != "normal" ? picOrDwg : uploadOthers;
      upload(formData)
        .then((res) => {
          this.$layer.close(loading);
          const { code, data, msg } = res.data;
          if (code && data) {
            this.$message.success(msg);
            this.panelShow(this.currentObj);
            this.fileUploadVisible = false;
          } else {
            this.$message.error(msg);
          }
        })
        .catch((error) => {
          this.$layer.close(loading);
          console.log(error);
        });
    },
    resCancel() {
      this.resShow = false;
    },
    resOk(data) {
      // 不走审批
      // this.submitShow = true;
      addMaterial(data).then((res) => {
        let { code, data, msg } = res.data;
        if (code && data) {
          this.$message.success("上传成功");
          this.resShow = false;
        } else {
          this.$message.error(msg);
        }
      });
    },
    successSubmit() {
      this.submitShow = false;
      this.$nextTick(() => {
        this.resShow = false;
      });
    },
    cancelSubmit() {
      this.submitShow = false;
    },
    dwgReview(url) {
      this.spinning = true;
      dwgReview(url)
        .then((res) => {
          this.spinning = false;
          let rs = res.data;
          if (rs.code == 1 && rs.data) {
            window.open(this.webUrl + rs.data);
          } else {
            this.$message.error(rs.msg);
          }
        })
        .catch((error) => {
          this.spinning = false;
          console.log(error);
        });
    },
    activeFN() {
      this.resize();
    },
    panelShow(obj) {
      const { id, dwgUrl, attachLocks, subModel } = obj;
      this.currentObj = obj;
      let reg = new RegExp("^" + subModel, "i");
      let reg2 = /(.*\\+)*(.*)$/;
      getFileList(id).then((res) => {
        const { code, data, data2 } = res.data;
        let list = [];
        if (code && data) {
          list = data2.filter((v) => {
            let fileName = v.match(reg2)[2];
            let arr = fileName.split(".");
            let fileType = arr.length ? arr[arr.length - 1].toLowerCase() : "";
            return !(reg.test(fileName) && fileType == "dwg");
          });
        }
        if (dwgUrl) {
          list.unshift(dwgUrl);
        }
        this.info = { attachLocks, id, oprateInfo: id };
        this.fileList = list;
        this.fileListShow = true;
      });
    },
    fileListCancel() {
      this.fileListShow = false;
    },
    goHistory(record) {
      const { rootModel } = record;
      this.$router.push({
        path: "/resource/material-history",
        query: { rootModel },
      });
    },
  },
  watch: {
    update(n) {
      if (-1 != n && !this._inactive) {
        this.$nextTick(() => {
          const table = this.$refs.table;
          const header = document.querySelectorAll(
            ".doc-center-table .ant-table-header"
          )[0].clientHeight;
          const bar = document.querySelectorAll(".header-bar")[0].clientHeight;
          if (table.fullScreen) {
            this.y = table.$el.clientHeight - bar - header - 64;
          } else {
            const wraper = this.$refs.wraper.clientHeight;
            const card = document.querySelectorAll(".ant-card-body")[0];
            const { paddingBottom, paddingTop } = getComputedStyle(card, null);
            const h =
              wraper -
              header -
              64 -
              bar -
              parseInt(paddingBottom) -
              parseInt(paddingTop);
            // console.log(h, "h",wraper, header, bar );
            this.y = h;
          }
        });
      }
    },
    affixed() {
      setTimeout(() => {
        this.update = Math.random();
      }, 200);
    },
  },
  mounted() {
    this.sendMessage();
    window.addEventListener("resize", this.resize);
  },
  destroyed() {
    window.removeEventListener("resize", this.resize);
  },
};
</script>

<style scoped lang="less">
.main {
  height: 100%;
  position: relative;
  .inner {
    position: absolute;
    left: 0;
    top: 0;
    right: 0;
    bottom: 0;
  }
}
.img-wraper {
  width: 80px;
  height: 50px;
  display: inline-block;
  .image-view {
    width: 100%;
    height: 100%;
    /deep/img {
      width: 100%;
      height: 100%;
      object-fit: contain;
    }
  }
}
/deep/table {
  table-layout: fixed;
}
.footer {
  padding-top: 10px;
  display: flex;
  justify-content: flex-end;
}
.type-grp {
  margin-bottom: 20px;
}
</style>
